'use strict';

var fs = require('fs');
var path = require('path');
var exec = require('child_process').exec;
var colors = require('colors');
var Q = require('q');

var currentDir = __dirname;

var read = require('read');

exports.name = 'init';
exports.desc = 'scaffold tools. Initialize a rake-zbj project';

var projectName, rapId;

var exists = function (filePath){
    try{
        fs.statSync(filePath);
    }catch(err){
        return false;
    }
    return true;
};

exports.register = function(commander) {

    commander
        .command('project')
        .description('create a rake-zbj project');


    commander.action(function () {
        run();
    });

};

function run(){

    promptProjectName()
    .then(function( result  ){
        projectName = result;
        promptRAPId()
        .then(function( result ){
            rapId = result;
        })
        .then(function(){
            console.log('\n');
            console.log('你的项目信息为:'.green);
            console.log('  项目名: ' + projectName);
            console.log('  rap id: ' + rapId);
            console.log('\n');
        })
        .then(promptConfirm)
        .then(function(){
                scaffoldProject( projectName, rapId );
            }, function(){
                run();
            });
    }, function( result ){
        run();
    })

}

function promptMaker( prompt ){
    var defer = Q.defer();
    read({prompt: prompt.gray}, function( err, result ){
        if ( err ) {
            defer.reject(new Error(err));
        } else {
            defer.resolve( result );
        }
    });
    return defer.promise;
}

function promptConfirm(){
    var confirmDefer = Q.defer();

    promptMaker('确定么[y|N|exit]：')
        .then(function( result ){
            if ( result.charAt(0).toLowerCase() == 'y' ) {
                confirmDefer.resolve( true );
            } else if ( result == 'exit' ) {
                process.exit(-1);
            } else {
                confirmDefer.reject( false );
            }
        });
    return confirmDefer.promise;
}

function promptProjectName(){
    var defer = Q.defer();

    promptMaker('请输入项目ID，如 login：')
        .then(function( result ){
            var projectName = result;
            var dirName = 'fis-' + result;
            var projectPath = path.join(process.cwd(), dirName);

            if( exists( projectPath ) ) {
                console.log(dirName + ' 已存在，请输入其它项目名');
                defer.reject( false );
            } else {
                defer.resolve( projectName );
            }
        });

    return defer.promise;
}


function promptRAPId(){
    return promptMaker('请输入rap ID，如 44：');
}

function scaffoldProject( projectName, rapId ){
    var child;
    child = exec('node ' + path.join(currentDir, './lib/rake-zbj-scaffold.js')  + ' ' + projectName + ' ' + rapId,
        function (error, stdout, stderr) {
            console.log( stdout );
            if (error !== null) {
                console.log('scaffold tool error: ' + error);
            }
        }
    );
}


