#!/usr/bin/env node

var program = require('commander');
var mkdirp = require('mkdirp');
var os = require('os');
var fs = require('fs');
var path = require('path');
var readline = require('readline');

var _exit = process.exit;
var eol = os.EOL;

var dirPrefix = 'fis-';

// Re-assign process.exit because of commander
// TODO: Switch to a different command framework
process.exit = exit

program.parse(process.argv);

//记录用户输入对象
var config={
    projectName : program.args[0],
    projectId : program.args[1]
};

if (!exit.exited) {
    main();
}


/**
 * Main program.
 */

function main() {
    // Path
    var destinationPath = '.';
    createApplication(config.projectName, destinationPath, setRapId);
}

/**
 * Create application at the given name `app_name`.
 *
 * @param {String} app_name
 */

// 创建项目目录
function createApplication(app_name,path,callback) {
    var dir_name = dirPrefix + app_name;
    var wait = 5;
    console.log();
    function complete() {
        if (--wait) return;
        console.log();
        console.log();
        process.stdin.destroy();

        // 创建目录完后的回调函数，重置fis-conf里的rapId
        callback(dir_name, config.projectId);
    }

    // Tpl
    var layout = loadTemplate('tpl/layout.tpl');
    var indexTpl = loadTemplate('tpl/index.tpl');

    // JavaScript
    var commonJs = loadTemplate('js/common.js');
    var fis = loadTemplate('js/fis-conf.js');
    var indexJs = loadTemplate('js/index.js');

    // CSS
    var commonLess = loadTemplate('css/common.less');
    var indexLess = loadTemplate('css/index.less');

    // 创建目录结构
    mkdir(dir_name, function () {
        mkdir(dir_name + '/page/common', function () {
            write(dir_name + '/page/common/layout.tpl', layout.replace('{app_name}', app_name));
            complete();
        });
        mkdir(dir_name + '/page/demo', function () {
            write(dir_name + '/page/demo/index.tpl', indexTpl.replace('{app_name}', app_name));
            complete();
        });

        mkdir(dir_name + '/static/common');
        mkdir(dir_name + '/static/lib');
        mkdir(dir_name + '/static/common', function () {
            write(dir_name + '/static/common/common.less', commonLess);
            write(dir_name + '/static/common/common.js', commonJs);
            complete();
        });
        mkdir(dir_name + '/static/demo', function () {
            write(dir_name + '/static/demo/index.less', indexLess);
            write(dir_name + '/static/demo/index.js', indexJs);
            complete();
        });

        mkdir(dir_name + '/widget/common');
        mkdir(dir_name + '/widget/ui/components');

        // write files
        write(dir_name + '/fis-conf.js', fis.replace('{app_name}', app_name));
        complete();
    });

}

// 重置fis-conf里的rapId
function setRapId(projectName,projectId){
    fs.readFile(projectName + '/fis-conf.js', function (err, data) {
        if(err) throw err;
        else{
            fs.writeFile(projectName + '/fis-conf.js', data.toString().replace('{ID}', projectId));
        }
    });
}

/**
 * Graceful exit for async STDIO
 */

function exit(code) {
    // flush output for Node.js Windows pipe bug
    // https://github.com/joyent/node/issues/6247 is just one bug example
    // https://github.com/visionmedia/mocha/issues/333 has a good discussion
    function done() {
        if (!(draining--)) _exit(code);
    }

    var draining = 0;
    var streams = [process.stdout, process.stderr];

    exit.exited = true;

    streams.forEach(function (stream) {
        // submit empty write request and wait for completion
        draining += 1;
        stream.write('', done);
    });

    done();
}

/**
 * Load template file.
 */

function loadTemplate(name) {
    return fs.readFileSync(path.join(__dirname,'templates', name), 'utf-8');
}


/**
 * echo str > path.
 *
 * @param {String} path
 * @param {String} str
 */

function write(path, str, mode) {
    fs.writeFileSync(path, str, {mode: mode || 0666});
    console.log('   \x1b[36mcreate\x1b[0m : ' + path);
}

/**
 * Mkdir -p.
 *
 * @param {String} path
 * @param {Function} fn
 */

function mkdir(path, fn) {
    mkdirp(path, 0755, function (err) {
        if (err) throw err;
        console.log('   \033[36mcreate\033[0m : ' + path);
        fn && fn();
    });
}
